/*
                                 mondesi.js

Relies heavily on functionality provided by WZ_JSGRAPHICS.JS (thanks Walter).

Plotting routines assume a 300w x 100h graph (yep, q&d).


COPYRIGHT
---------
Copyright (C) 2008 Mark G.Daniel
This program, comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under the
conditions of the GNU GENERAL PUBLIC LICENSE, version 3, or any later version.
http://www.gnu.org/licenses/gpl.txt


VERSION
-------
29-NOV-2008  MGD  initial
*/

// note: many storage items are instantiated by MONDESI.C

var mdsBarCol1 = "LimeGreen",
    mdsBarCol2 = "Chocolate",
    mdsBarColAve = "Yellow",
    mdsHistCol1 = "DarkGreen",
    mdsHistCol2 = "Sienna",
    mdsHistColAve = "GoldenRod",
    mdsObviousBtn = "Fuchsia";

var mdsNodeData = new Array();

var mdsRunning = false,
    mdsUpdateDisplay = false,
    mdsSetPrintable = false;

// initial start
// -------------

function mdsInitialStart ()
{
   mdsNodeDataIframe (true);
   mdsRunning = mdsUpdateDisplay = true;
}

// (re)start (or stop) the update interval
// ---------------------------------------

var mdsStartStopBgCol;
function mdsStartStop ()
{ 
   mdsNodeDataIframe (!mdsRunning);
   var btn = document.getElementById("start_stop_btn");
   if (mdsRunning)
   {
      // currently running - stop
      if (mdsStartStopBgCol == undefined)
         mdsStartStopBgCol = btn.style.backgroundColor;
      btn.style.backgroundColor = mdsObviousBtn;
      btn.value = "start";
      mdsResetAlert(false);
      // just in case the [print] button is hit whilst stopped
      mdsDrawPrintable(true);
      mdsRunning = mdsUpdateDisplay = false;
   }
   else
   {
      // currently stopped - start
      btn.style.backgroundColor = mdsStartStopBgCol;
      btn.value = "stop";
      mdsNodeZero();
      mdsDrawPrintable(false);
      mdsRunning = mdsUpdateDisplay = true;
   }
}

// (re)initialise per-node data
// ----------------------------

function mdsNodeInit (which,active)
{
   var which1 = "node_" + which;
   if (mdsNodeData[which1] == undefined)
   {
      mdsNodeData[which1] = new Object();
      mdsNodeData[which1].timer = null;
      mdsNodeData[which1].which = which;
      mdsNodeData[which1].title = which.toUpperCase();
      mdsNodeData[which1].active = active;
      mdsNodeData[which1].graphic = new jsGraphics(which1);
   }

   mdsNodeData[which1].time = new Date().getTime();
   mdsNodeData[which1].ave = mdsNodeData[which1].max =
      mdsNodeData[which1].val1 = mdsNodeData[which1].val2 =
      mdsNodeData[which1].raw = mdsNodeData[which1].scale = 0;
   mdsNodeData[which1].fixed = false;

   mdsBarGraph (which);
   mdsBarPaint (which);

   mdsHistInit (which);
   mdsHistGraph (which);
   mdsHistPaint (which);
}

// scale displays according to maximum datum
// -----------------------------------------

function mdsScale ()
{
   for (which1 in mdsNodeData)
   {
      if (mdsNodeData[which1].fixed) continue;
      var which = which1.substring(5);
      var data1 = mdsHistData["hist_"+which].data1;
      var max = 0;
      for (tdata in data1)
         if (data1[tdata] > max) max = data1[tdata];
      mdsNodeData[which1].max = mdsNodeData[which1].scale = max;
      mdsNodeData[which1].ave = mdsNodeData[which1].raw = 0;
      mdsNodeData[which1].time = new Date().getTime();
      mdsBarPlot(which);
      mdsHistPlot(which);
   }
}

// zero node and history data, reset graphs
// ----------------------------------------

function mdsNodeZero ()
{
   for (which1 in mdsNodeData)
   {
      var which = which1.substring(5);
      mdsNodeInit (which);
      mdsMonVal(which,-1,-1,-1,-1,0);
   }
}

// record a datum and graph its value
// ----------------------------------
// (called by the data streaming in the MONDESI hidden iframe)

function mdsMonVal (which,value1,value2,scale,raw,tstamp)
{
   if (!mdsRunning && tstamp != -1) return;

   var which1 = "node_" + which;
   if (mdsNodeData[which1] == undefined) return;

   if (!mdsNodeData[which1].active) return;

   if (raw >= 0 && mdsNodeData[which1].raw > raw)
   {
      // system restart?
      mdsNodeZero (which,null,null);
   }

   // arrange a timed update (must be done before 'scale' is adjusted)
   var updateGraph = "mdsMonVal('"+which+"','"+value1+"','"+
                                value2+"','"+scale+"','"+raw+"',0)";
   if (mdsNodeData[which1].timer != null)
      clearTimeout(mdsNodeData[which1].timer);
   mdsNodeData[which1].timer = setTimeout(updateGraph,mdsRefreshInterval*2000);

   if (raw >= 0 && mdsNodeData[which1].raw == 0) mdsNodeData[which1].raw = raw;
   if (value1 > mdsNodeData[which1].max) mdsNodeData[which1].max = value1;
   var max = mdsNodeData[which1].max;
   if (scale >= 0)
      mdsNodeData[which1].fixed = true;
   else
      scale = max;
   mdsNodeData[which1].scale = scale;

   var average = 0;
   if (raw >= 0)
   {
      // calculate average
      var time = new Date().getTime();
      var secs = (time - mdsNodeData[which1].time) / 1000;
      if (secs > 1)
         average = Math.round((raw - mdsNodeData[which1].raw) / secs);
   }
   mdsNodeData[which1].ave = average;
   mdsNodeData[which1].val1 = value1;
   mdsNodeData[which1].val2 = value2;

   mdsBarPlot(which);

   // history data
   mdsHistVal (which,value1,value2,tstamp)
}

// cancel any outstanding alert and optionally set a new one
// ---------------------------------------------------------

var mdsAlertTimer;
function mdsResetAlert (newAlert)
{
   if (mdsAlertTimer != undefined) clearTimeout(mdsAlertTimer);
   if (newAlert) mdsAlertTimer = setTimeout("mdsAlert()",mdsAlertSecs*1000);
}

// that the node has stopped being updated
// ---------------------------------------

function mdsAlert ()
{
   document.getElementById("node_notes").style.backgroundColor = "red";
   mdsResetAlert(false);
}

// send a message from a child iframe to the parent
// ------------------------------------------------

function mdsToMsgChannel (forEval)
{
   // MSIE(6) and SeaMonkey(1.6) (at least) deny permission for these
   try
   {
      var win = parent.window;
      if (win != undefined && typeof win.postMessage == "function")
         // message posting browsers (HTML5)
         win.postMessage (forEval, "*");
      else
         // inter-frame kludge
         mdsToBackChannel (forEval);
   }
   catch (err)
   {
      // inter-frame kludge
      mdsToBackChannel (forEval);
   }
}

// message passing via hidden iframe (HTML4)
// -----------------------------------------

var mdsBackChanCount = 0;
function mdsToBackChannel (forEval)
{
   var url = mdsRefererHost + "/mondesi/-/js/bchan.html#" + forEval;
   // MSIE(6+7) gets confused removing then adding elements with identical IDs!
   var iframe = document.getElementById("bchan_iframe_"+mdsBackChanCount);
   if (iframe != undefined)
   {
      mdsBody.removeChild(iframe);
      delete iframe;
   }
   iframe = document.createElement("iframe");
   iframe.setAttribute("id","bchan_iframe_"+(mdsBackChanCount++));
   iframe.setAttribute("src",url);
   iframe.setAttribute("height",0);
   iframe.setAttribute("width",0);
   iframe.setAttribute("frameBorder",0);
   mdsBody.appendChild(iframe);
}

// stop/start the streaming data flow (so that printing can occur)
// ---------------------------------------------------------------

var mdsDataIframeCount = 0;
var mdsDataIframe;
function mdsNodeDataIframe (start)
{
   if (mdsDataIframe != undefined)
   {
      // MSIE (6+7) seems to need this to dissociate from the data stream
      mdsDataIframe.src = "/mondesi/-/js/nl.html";

      mdsBody.removeChild(mdsDataIframe);
      delete mdsDataIframe;
      mdsDataIframe = undefined;
      mdsResetAlert (false);
   }
   if (start)
   {
      var url = mdsScriptName + "?" + mdsRefreshInterval;
      mdsDataIframe = document.createElement("iframe");
      // MSIE(6+7) gets confused removing then adding elements with same IDs!
      mdsDataIframe.setAttribute("id","node_iframe_"+mdsDataIframeCount);
      mdsDataIframe.setAttribute("src",url);
      mdsDataIframe.setAttribute("height",0);
      mdsDataIframe.setAttribute("width",0);
      mdsDataIframe.setAttribute("frameBorder",0);
      mdsBody.appendChild(mdsDataIframe);
      mdsResetAlert (true);
   }
}

// history button has been toggled
// -------------------------------

function mdsHistoryClick ()
{
   mdsHistVisible = !mdsHistVisible;
   var barHeight = document.getElementById("node_graph").scrollHeight;
   var histHeight = document.getElementById("history_graph").scrollHeight;
   var remotePC = "mdsToggleHistory('" + mdsIframeId + "'," +
                  mdsHistVisible + "," + barHeight + "," + histHeight + ")";
   mdsToMsgChannel(remotePC);
}

// a juggling act - stopping data streaming, printing, restarting data
// -------------------------------------------------------------------

function mdsPrintNode ()
{
   if (!mdsRunning)
      setTimeout("window.print()",100);
   else
   if (mdsUpdateDisplay)
   {
      mdsDrawPrintable(true);
      mdsNodeDataIframe(false);
      setTimeout("window.print()",100);
      setTimeout("mdsNodeDataIframe(true)",2500);
      mdsDrawPrintableTimer = setTimeout("mdsDrawPrintable(false)",45000);
   }
   else
   {
      mdsDrawPrintable(false);
      if (mdsDrawPrintableTimer != undefined)
         clearTimeout(mdsDrawPrintableTimer);
   }
}

// (re)set WZ printable flag and redraw to allow printing
// ------------------------------------------------------

var mdsDrawPrintableBgCol;
function mdsDrawPrintable (setPrint)
{
   mdsSetPrintable = setPrint;
   for (which1 in mdsNodeData)
   {
      var which = which1.substring(5);
      mdsBarPlot(which);
      mdsHistPlot(which);
   }
   var btn = document.getElementById("print_btn");
   if (mdsSetPrintable)
   {
      if (mdsDrawPrintableBgCol == undefined)
         mdsDrawPrintableBgCol = btn.style.backgroundColor;
      btn.style.backgroundColor = mdsObviousBtn;
      mdsUpdateDisplay = false;
   }
   else
   {
      btn.style.backgroundColor = mdsDrawPrintableBgCol;
      mdsUpdateDisplay = true;
   }
}

// initialise a single datum bar graph
// -----------------------------------

function mdsBarGraph (which)
{
   var which1 = "node_" + which;
   var jsgr = mdsNodeData[which1].graphic;
   jsgr.recreate();
   jsgr.setPrintable(mdsSetPrintable);
   jsgr.setFont ("arial","10px",Font.PLAIN); 
   if (mdsNodeData[which1].active)
      jsgr.setColor("white");
   else
      jsgr.setColor("#d3d3d3"); // "lightGray" - stupid MSIE
   jsgr.fillRect (0, 0, mdsBarWidth+2, mdsBarHeight);
}

// add the axes and title of a bar graph then paint
// ------------------------------------------------

function mdsBarPaint (which)
{
   var which1 = "node_" + which;
   var jsgr = mdsNodeData[which1].graphic;
   jsgr.setColor("black");
   jsgr.drawRect (0, 0, mdsBarWidth+2, mdsBarHeight);
   if (which == "cpu" && mdsCpuCount > 1)
      jsgr.drawString(mdsNodeData["node_"+which].title+"x"+mdsCpuCount,7,2);
   else
      jsgr.drawString(mdsNodeData[which1].title,7,2);
   jsgr.paint();
}

// render a single datum of the bar graph
// --------------------------------------

function mdsBarPlot (which)
{
   if (!mdsUpdateDisplay) return;

   var which1 = "node_" + which;
   if (mdsNodeData[which1] == undefined) return;

   mdsBarGraph(which);
   if (!mdsNodeData[which1].active)
   {
      mdsBarPaint(which);
      return;
   }

   var height = mdsBarHeight;

   var jsgr = mdsNodeData[which1].graphic;
   jsgr.setColor("white");
   jsgr.fillRect (0, 0, mdsBarWidth+2, mdsBarHeight+2);

   mdsBarString(jsgr,mdsBarWidth,mdsNodeData[which1].scale);

   var widthAve = mdsNodeData[which1].ave *
                  mdsBarWidth / mdsNodeData[which1].scale;
   if (widthAve > mdsBarWidth) widthAve = mdsBarWidth;
   var widthVal1 = mdsNodeData[which1].val1 *
                   mdsBarWidth / mdsNodeData[which1].scale;
   if (widthVal1 > mdsBarWidth) widthVal1 = mdsBarWidth;

   if (mdsNodeData[which1].val2 >= 0)
      var widthVal2 = mdsNodeData[which1].val2 *
                      mdsBarWidth / mdsNodeData[which1].scale;
   else
      var widthVal2 = 0;

   if (widthAve > 0 && widthAve > widthVal1 && widthAve > widthVal2)
   {
      jsgr.setColor(mdsBarColAve);
      jsgr.fillRect (1, 0, widthAve+1, height);
      if (widthAve > 60) mdsBarString(jsgr,widthAve,mdsNodeData[which1].ave);
   }

   if (widthVal1 > 0 && widthVal1 > widthVal2)
   {
      jsgr.setColor(mdsBarCol1);
      jsgr.fillRect (1, 0, widthVal1+1, height);
      if (widthVal1 > 60) mdsBarString(jsgr,widthVal1,mdsNodeData[which1].val1);
   }

   if (widthVal2 > 0)
   {
      jsgr.setColor(mdsBarCol2);
      jsgr.fillRect (1, 0, widthVal2+1, height);
      if (widthVal2 > 60) mdsBarString(jsgr,widthVal2,mdsNodeData[which1].val2);
   }

   mdsBarPaint(which);
}

// display a numeric value on the bar as a string
// ----------------------------------------------

function mdsBarString (jsgr,position,value)
{
   var len = value.toString().length;
   var xpos = position - 5 - (len * 8) + (len * 2);
   jsgr.setColor("black");
   jsgr.drawString(value.toString(),xpos,2);
}

// update the textual node information
// -----------------------------------

var mdsNodeNotesGraphic;

function mdsNodeNotes (text)
{
   if (!mdsUpdateDisplay) return;

   if (mdsNodeNotesGraphic == undefined)
      var jsgr = mdsNodeNotesGraphic = new jsGraphics("node_notes");
   else
      var jsgr = mdsNodeNotesGraphic;
   jsgr.recreate();
   jsgr.setPrintable(mdsSetPrintable);
   jsgr.setColor("black");
   jsgr.setFont ("arial","14px",Font.PLAIN); 
   jsgr.drawStringRect (text, 10, 10, mdsHistWidth, "left"); 
   jsgr.paint();

   mdsResetAlert(true);
}

// initialise history data collection and graph rendering
// ------------------------------------------------------

var mdsTimeDiff = 0;
var mdsHistData = new Array();

function mdsHistInit (which)
{
   var which1 = "hist_" + which;
   if (mdsHistData[which1] != undefined)
   {
      var jsgr = mdsHistData[which1].graphic.recreate();
      delete mdsHistData[which1].data1;
      delete mdsHistData[which1].data2;
      delete mdsHistData[which1];
   }
   else
      var jsgr;
   mdsHistData[which1] = new Object();
   mdsHistData[which1].data1 = new Array();
   mdsHistData[which1].data2 = new Array();
   if (jsgr == undefined)
   {
      mdsHistData[which1].graphic = jsgr = new jsGraphics(which1);
      mdsHistData[which1].graphic.setPrintable(mdsSetPrintable);
   }
   else
      mdsHistData[which1].graphic = jsgr;
   jsgr.setPrintable(mdsSetPrintable);
   mdsHistData[which1].visible = false;
}

// record a datum in a sparse (js hash) array
// ------------------------------------------

function mdsHistVal (which,value1,value2,tstamp)
{
   var which1 = "hist_" + which;
   var data1 = mdsHistData[which1].data1,
       data2 = mdsHistData[which1].data2;

   if (tstamp == 0)
   {
      value1 = value2 = -1;
      // manufacture a timestamp if a timed update
      tstamp = (new Date().getTime() / 1000) - mdsTimeDiff;
   }
   else
      mdsTimeDiff = (new Date().getTime() / 1000) - tstamp;

   var tdata = Math.round(tstamp).toString();
   if (value1 >= 0) data1[tdata] = value1;
   if (value2 >= 0) data2[tdata] = value2;

   // eliminate data more than the age of the graph
   var timeout = tstamp - mdsHistWidth - 30;
   var timeoutCount = 0;
   for (tdata in data1)
   {
      if (tdata < timeout)
      {
         // keep one that's timed-out for graphing in the far-left
         if (timeoutCount++)
         {
            delete data1[tdata];
            delete data2[tdata];
         }
      }
   }

   mdsHistPlot (which);
}

// initialise a history line graph
// -------------------------------

function mdsHistGraph (which)
{
   var which1 = "hist_" + which;
   var jsgr = mdsHistData[which1].graphic;
   jsgr.recreate();
   jsgr.setPrintable(mdsSetPrintable);
   jsgr.setFont ("arial","10px",Font.PLAIN); 
   if (mdsNodeData["node_"+which].active)
      jsgr.setColor("white");
   else
      jsgr.setColor("#d3d3d3"); // "lightGray" - stupid MSIE
   jsgr.fillRect (1, 1, mdsHistWidth+1, mdsHistHeight+1);
}

// provide the axes and title of the graph then display
// ----------------------------------------------------

function mdsHistPaint (which)
{
   var which1 = "hist_" + which;
   var jsgr = mdsHistData[which1].graphic;
   jsgr.setColor("black");
   jsgr.drawRect (0, 0, mdsHistWidth+2, mdsHistHeight+2);
   for (var x = 30; x < mdsHistWidth-2; x += 30)
   {
      if (x % 60)
         jsgr.drawLine (x+1, mdsHistHeight+1, x+1, mdsHistHeight+1-2);
      else
         jsgr.drawLine (x+1, mdsHistHeight+1, x+1, mdsHistHeight+1-5);
   }
   if (which == "cpu" && mdsCpuCount > 1)
      jsgr.drawString(mdsNodeData["node_"+which].title+"x"+mdsCpuCount,5,5);
   else
      jsgr.drawString(mdsNodeData["node_"+which].title,5,5);
   jsgr.paint();
}

// display the numeric scale and values on the plot
// ------------------------------------------------

function mdsHistScale (jsgr,value1,value2,scale)
{
   if (scale >= 0)
   {
      var len = scale.toString().length;
      var xpos = mdsHistWidth - 5 - (len * 8) + (len * 2);
      jsgr.setColor("black");
      jsgr.drawString(scale.toString(),xpos,5);
   }
   if (scale > 0 && value1 >= 0)
   {
      len = value1.toString().length;
      xpos = mdsHistWidth - 5 - (len * 8) + (len * 2);
      jsgr.setColor(mdsHistCol1);
      jsgr.drawString(value1.toString(),xpos,20);
   }
   if (scale > 0 && value1 > 0 && value2 >= 0)
   {
      len = value2.toString().length;
      xpos = mdsHistWidth - 5 - (len * 8) + (len * 2);
      jsgr.setColor(mdsHistCol2);
      jsgr.drawString(value2.toString(),xpos,35);
   }
}

// display an average on the plot
// ------------------------------

function mdsHistAverage (jsgr,value,scale)
{
   if (value <= 0 || value == NaN) return;
   var average = value * mdsHistHeight / scale;
   if (average <= 0) return;
   if (average > mdsHistHeight) average = mdsHistHeight;
   jsgr.setColor(mdsHistColAve);
   jsgr.drawLine (mdsHistWidth+1, 100-average+1, 1, 100-average+1);
   if (average < 12)
      jsgr.drawString(value,5,85-average+2);
   else
   if (average < 85)
      jsgr.drawString(value,5,100-average+2);
   else
      jsgr.drawString(value,35,100-average+2);
}

// plot a history graph from stored sparse data
// --------------------------------------------

function mdsHistPlot (which)
{
   var which1 = "hist_" + which;
   if (mdsHistData[which1] == undefined) return;

   if (!mdsHistVisible)
   {
      // obviously do not plot if not visible!
      if (mdsHistData[which1].visible)
      {
         mdsHistData[which1].visible = false;
         mdsHistData[which1].graphic.recreate();
         mdsHistData[which1].graphic.setPrintable(mdsSetPrintable);
      }
      return;
   }
   mdsHistData[which1].visible = true;

   if (!mdsUpdateDisplay) return;

   var data1 = mdsHistData[which1].data1,
       data2 = mdsHistData[which1].data2;

   tstamp = (new Date().getTime() / 1000) - mdsTimeDiff;

   mdsHistGraph(which);
   if (!mdsNodeData["node_"+which].active)
   {
      mdsHistPaint (which);
      return;
   }

   var jsgr = mdsHistData[which1].graphic;

   // wz canvas begins in the top-left, pen beneath and right of coords
   var scale = mdsNodeData["node_"+which].scale;

   mdsHistAverage(jsgr,mdsNodeData["node_"+which].ave,scale);

   // eliminate data more than the age of the graph
   var timeout = tstamp - mdsHistWidth - 30;
   var scale = mdsNodeData["node_"+which].scale;

   var x = y = lastx1 = lasty1 = lastx2 = lasty2 = 0;
   for (tdata in data1)
   {
      var apoint = parseInt(tdata);
      x = tstamp - apoint;
      if (x > mdsHistWidth) x = mdsHistWidth;

      y = data1[tdata] * mdsHistHeight / scale;
      if (y > mdsHistHeight) y = mdsHistHeight;
      jsgr.setColor(mdsHistCol1);
      if (lasty1 > 0)
         jsgr.drawLine (mdsHistWidth-lastx1, 100-lasty1+1,
                        mdsHistWidth-x, 100-lasty1+1);
      jsgr.drawLine (mdsHistWidth-x, 100-lasty1+1,
                     mdsHistWidth-x, 100-y+1);
      lastx1 = x;
      lasty1 = y;

      if (data2[tdata] != undefined)
      {
         y = data2[tdata] * mdsHistHeight / scale;
         if (y > mdsHistHeight) y = mdsHistHeight;
         jsgr.setColor(mdsHistCol2);
         if (lasty2 > 0)
            jsgr.drawLine (mdsHistWidth-lastx2, 100-lasty2+1,
                           mdsHistWidth-x, 100-lasty2+1);
         jsgr.drawLine (mdsHistWidth-x, 100-lasty2+1,
                        mdsHistWidth-x, 100-y+1);
         lastx2 = x;
         lasty2 = y;
      }
   }

   if (lasty1 > 0)
   {
      jsgr.setColor(mdsHistCol1);
      jsgr.drawLine (mdsHistWidth-lastx1+1, 100-lasty1+1,
                     mdsHistWidth+1, 100-lasty1+1);
   }
   if (lasty2 > 0)
   {
      jsgr.setColor(mdsHistCol2);
      jsgr.drawLine (mdsHistWidth-lastx2+1, 100-lasty2+1,
                     mdsHistWidth+1, 100-lasty2+1);
   }

   mdsHistScale(jsgr,mdsNodeData["node_"+which].val1,
                     mdsNodeData["node_"+which].val2,
                     mdsNodeData["node_"+which].scale);

   mdsHistPaint (which);
}

// fin!
